@@ -5,6 +5,7 @@
 #include <string>
 #include <time.h>
 
+#include "achievements.hpp"
 #include "asserts.hpp"
 #include "blur.hpp"
 #include "collision_utils.hpp"
@@ -18,6 +19,7 @@
 #include "filesystem.hpp"
 #include "formula_callable_definition.hpp"
 #include "formula_profiler.hpp"
+#include "i18n.hpp"
 #include "level.hpp"
 #include "level_runner.hpp"
 #include "object_events.hpp"
@@ -39,8 +41,10 @@
 #include "text_entry_widget.hpp"
 #include "thread.hpp"
 #include "unit_test.hpp"
+#include "wml_formula_adapter.hpp"
 #include "wml_parser.hpp"
 #include "wml_node.hpp"
+#include "wml_schema.hpp"
 #include "wml_utils.hpp"
 #include "wml_writer.hpp"
 #include "preferences.hpp"
@@ -147,7 +151,7 @@ FUNCTION_DEF(load_game, 0, 0, "load_game(): loads the saved game")
 	return variant(new load_game_command());
 END_FUNCTION_DEF(load_game)
 
-FUNCTION_DEF(can_load_game, 0, 0, "can_load_game(): returns true iff there is a saved game available to load")
+FUNCTION_DEF(can_load_game, 0, 0, "can_load_game(): returns true if there is a saved game available to load")
 	return variant(sys::file_exists(preferences::save_file_path()));
 END_FUNCTION_DEF(can_load_game)
 
@@ -410,32 +414,8 @@ public:
 		//This means objects are centered on the point they're spawned on, which is a lot more intuitive for scripting.
 		e->set_pos(e->x() - e->current_frame().width() / 2 , e->y() - e->current_frame().height() / 2);
 
-		if(!place_entity_in_level(lvl, *obj)) {
-			//the object can't immediately/easily be placed in the level
-			//due to a solid collision. Try to incrementally push it in
-			//different directions and try to place it until we find
-			//a direciton that works.
-			const int xpos = obj->x();
-			const int ypos = obj->y();
-
-			bool found = false;
-			for(int distance = 4; distance < 256 && !found; distance *= 2) {
-				const point points[] = { point(xpos-distance, ypos),
-				                         point(xpos+distance, ypos),
-				                         point(xpos, ypos-distance),
-				                         point(xpos, ypos+distance), };
-				foreach(const point& p, points) {
-					obj->set_pos(p);
-					if(place_entity_in_level(lvl, *obj)) {
-						found = true;
-						break;
-					}
-				}
-			}
-
-			if(!found) {
-				return;
-			}
+		if(!place_entity_in_level_with_large_displacement(lvl, *obj)) {
+			return;
 		}
 
 		lvl.add_character(e);
@@ -446,7 +426,7 @@ public:
 		//and let them record the child's details.
 		game_logic::map_formula_callable* spawn_callable(new game_logic::map_formula_callable);
 		variant holder(spawn_callable);
-		spawn_callable->add("parent", variant(&ob));
+		spawn_callable->add("spawner", variant(&ob));
 		spawn_callable->add("child", variant(obj));
 		ob.handle_event("child_spawned", spawn_callable);
 		obj->handle_event("spawned", spawn_callable);
@@ -484,9 +464,7 @@ FUNCTION_DEF(spawn_player, 4, 5, "spawn_player(string type_id, int midpoint_x, i
 END_FUNCTION_DEF(spawn_player)
 
 FUNCTION_DEF(object, 4, 5, "object(string type_id, int midpoint_x, int midpoint_y, int facing, (optional) map properties) -> object: constructs and returns a new object. Note that the difference between this and spawn is that spawn returns a command to actually place the object in the level. object only creates the object and returns it. It may be stored for later use.")
-	//generate a random number just so we mark this as being a
-	//function which shouldn't have its result cached.
-	rng::generate();
+	formula::fail_if_static_context();
 
 	const std::string type = args()[0]->evaluate(variables).as_string();
 	const int x = args()[1]->evaluate(variables).as_int();
@@ -786,6 +764,19 @@ FUNCTION_DEF(solid, 3, 5, "solid(level, int x, int y, (optional)int w=1, (option
 	return variant(lvl->solid(r));
 END_FUNCTION_DEF(solid)
 
+FUNCTION_DEF(standable, 3, 5, "standable(level, int x, int y, (optional)int w=1, (optional) int h=1) -> boolean: returns true iff the level contains standable space within the given (x,y,w,h) rectangle")
+	level* lvl = args()[0]->evaluate(variables).convert_to<level>();
+	const int x = args()[1]->evaluate(variables).as_int();
+	const int y = args()[2]->evaluate(variables).as_int();
+
+	int w = args().size() >= 4 ? args()[3]->evaluate(variables).as_int() : 1;
+	int h = args().size() >= 5 ? args()[4]->evaluate(variables).as_int() : 1;
+
+	rect r(x, y, w, h);
+
+	return variant(lvl->standable(r));
+END_FUNCTION_DEF(standable)
+
 class set_solid_command : public entity_command_callable {
 	rect r_;
 	bool is_solid_;
@@ -1040,7 +1031,7 @@ private:
 			if(var.is_callable()) {
 				const_entity_ptr e = var.try_convert<entity>();
 				if(e) {
-					std::cerr << "set speaker...\n";
+					//std::cerr << "set speaker...\n";
 					d->set_speaker_and_flip_side(e);
 				}
 
@@ -1109,6 +1100,20 @@ private:
 						}
 #endif
 						switch(event.type) {
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+						// make sure nothing happens while the app is supposed to be "inactive"
+						case SDL_WINDOWEVENT:
+						if (event.window.event == SDL_WINDOWEVENT_MINIMIZED)
+						{
+							SDL_Event e;
+							while (SDL_WaitEvent(&e))
+							{
+								if (e.type == SDL_WINDOWEVENT && e.window.event == SDL_WINDOWEVENT_RESTORED)
+									break;
+							}
+						}
+						break;
+#endif
 						case SDL_QUIT:
 							throw interrupt_game_exception();
 						case SDL_USEREVENT:
@@ -1196,6 +1201,29 @@ FUNCTION_DEF(end_game, 0, 0, "end_game(): exits the game")
 	return variant(new end_game_command());
 END_FUNCTION_DEF(end_game)
 
+class achievement_command : public entity_command_callable
+{
+public:
+	explicit achievement_command(const std::string& str) : str_(str)
+	{}
+
+	virtual void execute(level& lvl, entity& ob) const {
+		if(attain_achievement(str_)) {
+			achievement_ptr a = achievement::get(str_);
+			if(a) {
+				sound::play("achievement-attained.wav");
+				set_displayed_achievement(a);
+			}
+		}
+	}
+private:
+	std::string str_;
+};
+
+FUNCTION_DEF(achievement, 1, 1, "achievement(id): unlocks the achievement with the given id")
+	return variant(new achievement_command(args()[0]->evaluate(variables).as_string()));
+END_FUNCTION_DEF(achievement)
+
 class debug_command : public entity_command_callable
 {
 public:
@@ -1216,14 +1244,30 @@ FUNCTION_DEF(debug, 1, -1, "debug(...): outputs arguments to the console")
 
 	std::string str;
 	for(int n = 0; n != args().size(); ++n) {
+		if(n > 0) {
+			str += " ";
+		}
+
 		str += args()[n]->evaluate(variables).to_debug_string();
 	}
 
-	fprintf(stderr, "DEBUG FUNCTION: %s\n", str.c_str());
+	//fprintf(stderr, "DEBUG FUNCTION: %s\n", str.c_str());
 
 	return variant(new debug_command(str));
 END_FUNCTION_DEF(debug)
 
+FUNCTION_DEF(debug_fn, 2, 2, "debug(msg, expr): evaluates and returns expr. Will print 'msg' to stderr")
+	variant res = args()[1]->evaluate(variables);
+	if(preferences::debug()) {
+		variant msg = args()[0]->evaluate(variables);
+		std::string s = msg.to_debug_string();
+		debug_console::add_message(s);
+		std::cerr << "CONSOLE: " << s << "\n";
+	}
+
+	return res;
+END_FUNCTION_DEF(debug_fn)
+
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE && !TARGET_OS_MAEMO
 class debug_console_command : public entity_command_callable
 {
@@ -1328,11 +1372,20 @@ FUNCTION_DEF(fire_event, 1, 3, "fire_event((optional) object target, string id,
 	const_formula_callable_ptr callable;
 
 	if(args().size() == 3) {
-		target = args()[0]->evaluate(variables).convert_to<entity>();
+		variant v1 = args()[0]->evaluate(variables);
+		if(v1.is_null()) {
+			return variant();
+		}
+
+		target = v1.convert_to<entity>();
 		event = args()[1]->evaluate(variables).as_string();
 		callable = args()[2]->evaluate(variables).as_callable();
 	} else if(args().size() == 2) {
 		variant v1 = args()[0]->evaluate(variables);
+		if(v1.is_null()) {
+			return variant();
+		}
+
 		variant v2 = args()[1]->evaluate(variables);
 		if(v1.is_string()) {
 			event = v1.as_string();
@@ -1462,10 +1515,6 @@ public:
 	virtual void execute(level& lvl, entity& ob) const {
 		level::portal p;
 		p.level_dest = level_;
-		if(p.level_dest.empty()) {
-			p.level_dest = lvl.id();
-		}
-
 		p.dest_starting_pos = true;
 		p.dest_label = label_;
 		p.automatic = true;
@@ -1701,23 +1750,25 @@ END_FUNCTION_DEF(blur)
 
 class text_command : public custom_object_command_callable {
 public:
-	text_command(const std::string& text, const std::string& font, int size)
-	  : text_(text), font_(font), size_(size) {
+	text_command(const std::string& text, const std::string& font, int size, bool centered)
+	  : text_(text), font_(font), size_(size), centered_(centered) {
 	}
 
 	virtual void execute(level& lvl, custom_object& ob) const {
-		ob.set_text(text_, font_, size_*2);
+		ob.set_text(text_, font_, size_*2, centered_);
 	}
 private:
 	std::string text_, font_;
 	int size_;
+	bool centered_;
 };
 
-FUNCTION_DEF(text, 1, 3, "text(string text, (optional)string font='default', (optional)int size=1): adds text for the current object")
+FUNCTION_DEF(text, 1, 4, "text(string text, (optional)string font='default', (optional)int size=1, (optional)bool centered=false): adds text for the current object")
 	const std::string text = args()[0]->evaluate(variables).as_string();
 	const std::string font = args().size() > 1 ? args()[1]->evaluate(variables).as_string() : "default";
 	const int size = args().size() > 2 ? args()[2]->evaluate(variables).as_int() : 1;
-	return variant(new text_command(text, font, size));
+	const bool centered = args().size() > 3 ? args()[3]->evaluate(variables).as_bool() : false;
+	return variant(new text_command(text, font, size, centered));
 END_FUNCTION_DEF(text)
 
 FUNCTION_DEF(swallow_event, 0, 0, "swallow_event(): when used in an instance-specific event handler, this causes the event to be swallowed and not passed to the object's main event handler.")
@@ -1772,6 +1823,59 @@ FUNCTION_DEF(cosmic_shift, 2, 2, "cosmic_shift(int xoffset, int yoffet): adjust
 	return variant(new shift_level_position_command(args()[0]->evaluate(variables).as_int(), args()[1]->evaluate(variables).as_int()));
 END_FUNCTION_DEF(cosmic_shift)
 
+namespace {
+bool consecutive_periods(char a, char b) {
+	return a == '.' && b == '.';
+}
+}
+
+FUNCTION_DEF(eval, 1, 2, "eval(string formula, [obj context]): evaluate the given formula in the given context")
+	std::map<std::string, game_logic::formula_ptr> cache;
+	const std::string fml = args()[0]->evaluate(variables).as_string();
+	game_logic::formula_ptr& f = cache[fml];
+	if(f.get() == NULL) {
+		f = game_logic::formula::create_optional_formula(fml);
+	}
+
+	ASSERT_LOG(f.get() != NULL, "INVALID FORMULA PASSED TO EVAL: " << fml);
+
+	if(args().size() > 1) {
+		return f->execute(*args()[1]->evaluate(variables).as_callable());
+	} else {
+		return f->execute();
+	}
+END_FUNCTION_DEF(eval)
+
+FUNCTION_DEF(get_document, 1, 2, "get_document(string doc, [string schema]): return reference to the given WML document, optionally using the given schema")
+	const std::string docname = args()[0]->evaluate(variables).as_string();
+	std::string schema;
+	if(args().size() > 1) {
+		schema = args()[1]->evaluate(variables).as_string();
+	}
+
+	static std::map<std::pair<std::string,std::string>, variant> cache;
+	std::pair<std::string, std::string> key(docname, schema);
+	variant& v = cache[key];
+	if(v.is_null() == false) {
+		return v;
+	}
+
+	ASSERT_LOG(docname.empty() == false, "DOCUMENT NAME GIVEN TO get_document() IS EMPTY");
+	ASSERT_LOG(docname[0] != '/', "DOCUMENT NAME BEGINS WITH / " << docname);
+	ASSERT_LOG(std::adjacent_find(docname.begin(), docname.end(), consecutive_periods) == docname.end(), "DOCUMENT NAME CONTAINS ADJACENT PERIODS " << docname);
+
+	const wml::schema* sch = NULL;
+	if(schema.empty() == false) {
+		sch = wml::schema::get(schema);
+		ASSERT_LOG(sch != NULL, "COULD NOT FIND SCHEMA: " << schema);
+	}
+
+	const wml::node_ptr doc = wml::parse_wml_from_file(docname, sch);
+
+	v = variant(new wml::node_callable(doc));
+	return v;
+END_FUNCTION_DEF(get_document)
+
 class custom_object_function_symbol_table : public function_symbol_table
 {
 public:
@@ -1818,7 +1922,14 @@ void init_custom_object_functions(wml::const_node_ptr node)
 		const std::string& name = i1->second->attr("name");
 		std::vector<std::string> args = util::split(i1->second->attr("args"));
 
-		game_logic::formula_callable_definition_ptr args_definition = game_logic::create_formula_callable_definition(&args[0], &args[0] + args.size());
+		const std::string* first_arg = NULL;
+		const std::string* last_arg = NULL;
+		if(!args.empty()) {
+			first_arg = &args[0];
+			last_arg = &args[0] + args.size();
+		}
+
+		game_logic::formula_callable_definition_ptr args_definition = game_logic::create_formula_callable_definition(first_arg, last_arg);
 
 		recursive_function_symbol_table recursive_symbols(name, args, &get_custom_object_functions_symbol_table());
 		const_formula_ptr fml(new formula(i1->second->attr("formula"), &recursive_symbols, args_definition.get()));
