@@ -348,25 +348,25 @@ custom_object_type::custom_object_type(wml::const_node_ptr node, const custom_ob
 	is_vehicle_(wml::get_bool(node, "vehicle", false)),	
 	passenger_x_(wml::get_int(node, "passenger_x")),
 	passenger_y_(wml::get_int(node, "passenger_y")),
-	feet_width_(wml::get_int(node, "feet_width", 5)),
+	feet_width_(wml::get_int(node, "feet_width", 0)),
 	use_image_for_collisions_(wml::get_bool(node, "use_image_for_collisions", false)),
 	has_feet_(wml::get_bool(node, "has_feet", true) && use_image_for_collisions_ == false),
 	adjust_feet_on_animation_change_(wml::get_bool(node, "adjust_feet_on_animation_change", true)),
 	teleport_offset_x_(wml::get_int(node, "teleport_offset_x")),
 	teleport_offset_y_(wml::get_int(node, "teleport_offset_y")),
+	no_move_to_standing_(wml::get_bool(node, "no_move_to_standing")),
 	solid_(solid_info::create(node)),
 	platform_(solid_info::create_platform(node)),
 	has_solid_(solid_ || use_image_for_collisions_),
 	solid_dimensions_(has_solid_ || platform_ ? 0xFFFFFFFF : 0),
 	collide_dimensions_(0xFFFFFFFF),
 	weak_solid_dimensions_(has_solid_ || platform_ ? 0xFFFFFFFF : 0),
-	weak_collide_dimensions_(0xFFFFFFFF)
+	weak_collide_dimensions_(0xFFFFFFFF),
+	activation_border_(wml::get_int(node, "activation_border", 100))
 {
-#ifndef NO_EDITOR
 	if(node->get_child("editor_info")) {
 		editor_info_.reset(new editor_entity_info(node->get_child("editor_info")));
 	}
-#endif
 
 	const bool is_variation = base_type != NULL;
 
@@ -499,7 +499,7 @@ custom_object_type::custom_object_type(wml::const_node_ptr node, const custom_ob
 		entry->type_definition = entry->type_definition_holder.get();
 	}
 
-	std::cerr << "TMP_VARIABLES: '" << id_ << "' -> " << tmp_variables_.size() << "\n";
+	//std::cerr << "TMP_VARIABLES: '" << id_ << "' -> " << tmp_variables_.size() << "\n";
 
 	consts_.reset(new game_logic::map_formula_callable);
 	wml::const_node_ptr consts = node->get_child("consts");
